﻿module CassetteSpikesModule

using ..Ahorn, Maple

@mapdef Entity "altlevelshelper/cassettespikesup" CassetteSpikeUp(x::Integer, y::Integer, width::Integer=8, type::String="default", cassetteIndex::Integer=0)
@mapdef Entity "altlevelshelper/cassettespikesdown" CassetteSpikeDown(x::Integer, y::Integer, width::Integer=8, type::String="default", cassetteIndex::Integer=0)
@mapdef Entity "altlevelshelper/cassettespikesleft" CassetteSpikeLeft(x::Integer, y::Integer, height::Integer=8, type::String="default", cassetteIndex::Integer=0)
@mapdef Entity "altlevelshelper/cassettespikesright" CassetteSpikeRight(x::Integer, y::Integer, height::Integer=8, type::String="default", cassetteIndex::Integer=0)

CassetteSpikesUnion = Union{CassetteSpikeUp, CassetteSpikeDown, CassetteSpikeLeft, CassetteSpikeRight}

directions = Dict{String, String}(
    "altlevelshelper/cassettespikesup" => "up",
    "altlevelshelper/cassettespikesdown" => "down",
    "altlevelshelper/cassettespikesleft" => "left",
    "altlevelshelper/cassettespikesright" => "right",
)

Ahorn.editingOptions(entity::CassetteSpikesUnion) = Dict{String, Any}(
    "type" => Maple.spike_types
)

colorNames = Dict{String, Int}(
    "Blue" => 0,
    "Rose" => 1,
    "Bright Sun" => 2,
    "Malachite" => 3
)

const placements = Ahorn.PlacementDict()

for type in Maple.spike_types
	for (color, index) in colorNames
		key = "Cassette Spikes (Up) ($(uppercasefirst(type))) ($(index + 1) - $color) (Alt Levels Helper)"
		placements[key] = Ahorn.EntityPlacement(
			CassetteSpikeUp,
			"rectangle",
			Dict{String, Any}(
				"cassetteIndex" => index,
				"type" => type,
			)
		)

		key = "Cassette Spikes (Down) ($(uppercasefirst(type))) ($(index + 1) - $color) (Alt Levels Helper)"
		placements[key] = Ahorn.EntityPlacement(
			CassetteSpikeDown,
			"rectangle",
			Dict{String, Any}(
				"cassetteIndex" => index,
				"type" => type,
			)
		)

		key = "Cassette Spikes (Left) ($(uppercasefirst(type))) ($(index + 1) - $color) (Alt Levels Helper)"
		placements[key] = Ahorn.EntityPlacement(
			CassetteSpikeLeft,
			"rectangle",
			Dict{String, Any}(
				"cassetteIndex" => index,
				"type" => type,
			)
		)

		key = "Cassette Spikes (Right) ($(uppercasefirst(type))) ($(index + 1) - $color) (Alt Levels Helper)"
		placements[key] = Ahorn.EntityPlacement(
			CassetteSpikeRight,
			"rectangle",
			Dict{String, Any}(
				"cassetteIndex" => index,
				"type" => type,
			)
		)
	end
end

offsets = Dict{String, Tuple{Integer, Integer}}(
    "up" => (4, -4),
    "down" => (4, 4),
    "left" => (-4, 4),
    "right" => (4, 4),
)

rotations = Dict{String, Number}(
    "up" => 0,
    "right" => pi / 2,
    "down" => pi,
    "left" => pi * 3 / 2
)

rotationOffsets = Dict{String, Tuple{Number, Number}}(
    "up" => (0.5, 0.25),
    "right" => (1, 0.675),
    "down" => (1.5, 1.125),
    "left" => (0, 1.675)
)

resizeDirections = Dict{String, Tuple{Bool, Bool}}(
    "up" => (true, false),
    "down" => (true, false),
    "left" => (false, true),
    "right" => (false, true),
)

function Ahorn.renderSelectedAbs(ctx::Ahorn.Cairo.CairoContext, entity::CassetteSpikesUnion)
    direction = get(directions, entity.name, "up")
    theta = rotations[direction] - pi / 2

    width = Int(get(entity.data, "width", 0))
    height = Int(get(entity.data, "height", 0))

    x, y = Ahorn.position(entity)
    cx, cy = x + floor(Int, width / 2) - 8 * (direction == "left"), y + floor(Int, height / 2) - 8 * (direction == "up")

    Ahorn.drawArrow(ctx, cx, cy, cx + cos(theta) * 24, cy + sin(theta) * 24, Ahorn.colors.selection_selected_fc, headLength=6)
end

function Ahorn.selection(entity::CassetteSpikesUnion)
	x, y = Ahorn.position(entity)

	width = Int(get(entity.data, "width", 8))
	height = Int(get(entity.data, "height", 8))

	direction = get(directions, entity.name, "up")

	ox, oy = offsets[direction]

	return Ahorn.Rectangle(x + ox - 4, y + oy - 4, width, height)
end

function Ahorn.minimumSize(entity::CassetteSpikesUnion)
	return (8, 8)
end

function Ahorn.resizable(entity::CassetteSpikesUnion)
	direction = get(directions, entity.name, "up")

	return resizeDirections[direction]
end

function Ahorn.render(ctx::Ahorn.Cairo.CairoContext, entity::CassetteSpikesUnion)
    variant = get(entity.data, "type", "default")
    if haskey(directions, entity.name)
        direction = get(directions, entity.name, "up")

		#x, y = Ahorn.position(entity)
		#width = Int(get(entity.data, "width", 8))
		#height = Int(get(entity.data, "height", 8))
		#cx, cy = x + floor(Int, width / 2) - 8 * (direction == "left"), y + floor(Int, height / 2) - 8 * (direction == "up")
		
		#cstIndex = get(entity.data, "cassetteIndex", 0)
		#cstColor = (73, 170, 240) ./ 255

		#if cstIndex == 1
		#	cstColor = (240, 73, 190) ./ 255
		#end

		#if cstIndex == 2
		#	cstColor = (252, 220, 58) ./ 255
		#end

		#if cstIndex == 3
		#	cstColor = (56, 224, 78) ./ 255
		#end

		#Ahorn.setSourceColor(ctx, cstColor)
		#Ahorn.drawRectangle(ctx, cx, cy, width, height, cstColor)
		
        if variant == "tentacles"
			width = get(entity.data, "width", 16)
			height = get(entity.data, "height", 16)

			for ox in 0:16:width - 16, oy in 0:16:height - 16
				drawX, drawY = (ox, oy) .+ (16, 16) .* rotationOffsets[direction]
				Ahorn.drawSprite(ctx, "danger/tentacles00", drawX, drawY, rot=rotations[direction])
			end

			if width / 8 % 2 == 1 || height / 8 % 2 == 1
				drawX, drawY = (width - 16, height - 16) .+ (16, 16)
				Ahorn.drawSprite(ctx, "danger/tentacles00", drawX, drawY, rot=rotations[direction])
			end
        else        
            width = get(entity.data, "width", 8)
            height = get(entity.data, "height", 8)

            for ox in 0:8:width - 8, oy in 0:8:height - 8
                drawX, drawY = (ox, oy) .+ offsets[direction]
                Ahorn.drawSprite(ctx, "danger/spikes/$(variant)_$(direction)00", drawX, drawY)
            end
        end
    end
end

#function Ahorn.render(ctx::Ahorn.Cairo.CairoContext, entity::CassetteSpikesUnion)
#        direction = get(directions, entity.name, "up")      
#		width = get(entity.data, "width", 8)
#		height = get(entity.data, "height", 8)
#
#		for ox in 0:8:width - 8, oy in 0:8:height - 8
#			drawX, drawY = (ox, oy) .+ offsets[direction]
#			Ahorn.drawSprite(ctx, "objects/FactoryHelper/CassetteSpike/Static_$(direction)00", drawX, drawY)
#		end
#end

end
