﻿module CustommmmZipMover

using ..Ahorn, Maple, Cairo

@mapdef Entity "altlevelshelper/customzipmover" CustomizableZipMover(x::Integer, y::Integer, Theme::String="Normal", MovingBehaviour::String="Zip",
	TargetNodeBehaviour::String="SecondNode", CassetteMode::String="None", LinkColor::String="None", MovingSpeedMultiplier::Number=1.0,
	ReturningSpeedMultiplier::Number=1.0, CooldownAfterMoving::Number=0.5, CooldownAfterReturning::Number=0.5)

zip_mover_themes = String[
	"Normal", "Moon", "Colored", "Cassette"
]

zip_mover_moves = String[
	"Zip", "Chain", "Swap"
]

zip_mover_targets = String[
	"SecondNode", "NextNodeAndReverse", "NextNodeAndLoop"
]

zip_mover_cassettemodes = String[
	"None", "Blue", "Rose", "BrightSun", "Malachite"
]

cassetteColors = Dict{String, String}(
	"blue" => "49aaf0",
	"rose" => "f049be",
	"brightsun" => "fcdc3a",
	"malachite" => "38e04e",
)

const placements = Ahorn.PlacementDict()

for theme in zip_mover_themes
	for move in zip_mover_moves
		key = "Customizable Zip Mover ($(uppercasefirst(theme))) ($(uppercasefirst(move))) (Alt Levels Helper)"

		cstMode = "None"
		if theme == "Cassette"
			cstMode = "Blue"
		end

		placements[key] = Ahorn.EntityPlacement(
			CustomizableZipMover,
			"rectangle",
			Dict{String, Any}(
				"Theme" => theme,
				"MovingBehaviour" => move,
				"CassetteMode" => cstMode,
			),
			function(entity)
				if lowercase(get(entity.data, "MovingBehaviour", "Zip")) == "chain"
					entity.data["nodes"] = [(Int(entity.data["x"]) + Int(entity.data["width"]), Int(entity.data["y"])), (Int(entity.data["x"]) + Int(entity.data["width"]) * 4, Int(entity.data["y"]))]
				else
					entity.data["nodes"] = [(Int(entity.data["x"]) + Int(entity.data["width"]) + 8, Int(entity.data["y"]))]
				end
			end
		)
	end
end

Ahorn.editingOptions(entity::CustomizableZipMover) = Dict{String, Any}(
    "Theme" => zip_mover_themes,
	"MovingBehaviour" => zip_mover_moves,
	"TargetNodeBehaviour" => zip_mover_targets,
	"CassetteMode" => zip_mover_cassettemodes
)

Ahorn.nodeLimits(entity::CustomizableZipMover) = -1, -1
Ahorn.minimumSize(entity::CustomizableZipMover) = 16, 16
Ahorn.resizable(entity::CustomizableZipMover) = true, true

function Ahorn.selection(entity::CustomizableZipMover)
	nodes = get(entity.data, "nodes", ())
    x, y = Ahorn.position(entity)

    width = Int(get(entity.data, "width", 8))
    height = Int(get(entity.data, "height", 8))
	
	res = Ahorn.Rectangle[Ahorn.Rectangle(x, y, width, height)]
	
	for node in nodes
        nx, ny = Int.(node)

        push!(res, Ahorn.Rectangle(nx + floor(Int, width / 2) - 5, ny + floor(Int, height / 2) - 5, 10, 10))
    end

    return res
end

function getTextures(entity::CustomizableZipMover)
    theme = lowercase(get(entity, "Theme", "normal"))
    
    if theme == "moon"
		return "objects/zipmover/moon/block", "objects/zipmover/moon/light01", "objects/zipmover/moon/cog"
    end
	if theme == "normal"
		return "objects/zipmover/block", "objects/zipmover/light01", "objects/zipmover/cog"
	end
	return "objects/AltLevels/customzipmover/edges", "objects/zipmover/light01", "objects/zipmover/cog"
end

cassetteColors = Dict{String, String}(
	"blue" => "49aaf0",
	"rose" => "f049be",
	"brightsun" => "fcdc3a",
	"malachite" => "38e04e",
)

function renderCustomizableZipMover(ctx::Ahorn.Cairo.CairoContext, entity::CustomizableZipMover)
	colorCode = lowercase(get(entity.data, "LinkColor", "none"))
	linkColor = Ahorn.argb32ToRGBATuple(parse(Int, "ffffff", base=16))[1:3] ./ 255
	theme = lowercase(get(entity, "Theme", "normal"))
	ropeColor = (102, 57, 49) ./ 255

	drawCustomBorders = false

	if theme == "cassette"
		linkColor = Ahorn.argb32ToRGBATuple(parse(Int, cassetteColors[lowercase(get(entity.data, "CassetteMode", "blue"))], base=16))[1:3] ./ 255
		linkColor = (linkColor[1], linkColor[2], linkColor[3], 1.0)
		ropeColor = linkColor
		drawCustomBorders = true
	end

	if colorCode != "none"
		ropeColor = Ahorn.argb32ToRGBATuple(parse(Int, colorCode, base=16))[1:3] ./ 255
		ropeColor = (ropeColor..., 1.0)

		if theme == "colored"
			linkColor = Ahorn.argb32ToRGBATuple(parse(Int, colorCode, base=16))[1:3] ./ 255
			linkColor = (linkColor[1], linkColor[2], linkColor[3], 1.0)
			drawCustomBorders = true
		end
	end

	width = Int(get(entity.data, "width", 32))
	height = Int(get(entity.data, "height", 32))
	block, light, cog = getTextures(entity)
	lightSprite = Ahorn.getSprite(light, "Gameplay")
	tilesWidth = div(width, 8)
	tilesHeight = div(height, 8)
	nodes = deepcopy(entity.data["nodes"])
	pushfirst!(nodes, Ahorn.position(entity))
	stopPoint = length(nodes) - 1
	for node in 1:stopPoint
		x, y = Int.(nodes[node])
		nx, ny = Int.(nodes[node + 1])

		cx, cy = x + width / 2, y + height / 2
		cnx, cny = nx + width / 2, ny + height / 2

		length = sqrt((x - nx)^2 + (y - ny)^2)
		theta = atan(cny - cy, cnx - cx)

		Ahorn.Cairo.save(ctx)

		Ahorn.translate(ctx, cx, cy)
		Ahorn.rotate(ctx, theta)

		Ahorn.setSourceColor(ctx, ropeColor)
		Ahorn.set_antialias(ctx, 1)
		Ahorn.set_line_width(ctx, 1);

		# Offset for rounding errors
		Ahorn.move_to(ctx, 0, 4 + (theta <= 0))
		Ahorn.line_to(ctx, length, 4 + (theta <= 0))

		Ahorn.move_to(ctx, 0, -4 - (theta > 0))
		Ahorn.line_to(ctx, length, -4 - (theta > 0))

		Ahorn.stroke(ctx)
		
		Ahorn.Cairo.restore(ctx)
		
		Ahorn.drawSprite(ctx, cog, cnx, cny)
	end
	
	x, y = Ahorn.position(entity)
	
	Ahorn.drawRectangle(ctx, x + 2, y + 2, width - 4, height - 4, (0.0, 0.0, 0.0, 1.0))

	blockSprite = Ahorn.getSprite(block, "Gameplay")
	
	if drawCustomBorders
		#drawTinted(ctx, blockSprite, x,	y, 0, 0, 8, 8, linkColor)
		#drawTinted(ctx,	blockSprite, x, y, 0, 0, 8, 8, linkColor)
		#drawTinted(ctx, blockSprite, x + width - 8, y, 16, 0, 8, 8, linkColor)
		#drawTinted(ctx, blockSprite, x, y + height - 8, 0, 16, 8, 8, linkColor)

		for i in 2:tilesWidth - 1
			drawTinted(ctx, blockSprite, x + (i - 1) * 8, y, 8, 0, 8, 8, linkColor)
			drawTinted(ctx, blockSprite, x + (i - 1) * 8, y + height - 8, 8, 16, 8, 8, linkColor)
		end

		for i in 2:tilesHeight - 1
			drawTinted(ctx, blockSprite, x, y + (i - 1) * 8, 0, 8, 8, 8, linkColor)
			drawTinted(ctx, blockSprite, x + width - 8, y + (i - 1) * 8, 16, 8, 8, 8, linkColor)
		end

		drawTinted(ctx, blockSprite, x, y, 0, 0, 8, 8, linkColor)
		drawTinted(ctx, blockSprite, x + width - 8, y, 16, 0, 8, 8, linkColor)
		drawTinted(ctx, blockSprite, x, y + height - 8, 0, 16, 8, 8, linkColor)
		drawTinted(ctx, blockSprite, x + width - 8, y + height - 8, 16, 16, 8, 8, linkColor)
	else
		for i in 2:tilesWidth - 1
			Ahorn.drawImage(ctx, blockSprite, x + (i - 1) * 8, y, 8, 0, 8, 8)
			Ahorn.drawImage(ctx, blockSprite, x + (i - 1) * 8, y + height - 8, 8, 16, 8, 8)
		end

		for i in 2:tilesHeight - 1
			Ahorn.drawImage(ctx, blockSprite, x, y + (i - 1) * 8, 0, 8, 8, 8)
			Ahorn.drawImage(ctx, blockSprite, x + width - 8, y + (i - 1) * 8, 16, 8, 8, 8)
		end

		Ahorn.drawImage(ctx, blockSprite, x, y, 0, 0, 8, 8)
		Ahorn.drawImage(ctx, blockSprite, x + width - 8, y, 16, 0, 8, 8)
		Ahorn.drawImage(ctx, blockSprite, x, y + height - 8, 0, 16, 8, 8)
		Ahorn.drawImage(ctx, blockSprite, x + width - 8, y + height - 8, 16, 16, 8, 8)
	end

	Ahorn.drawImage(ctx, lightSprite, x + floor(Int, (width - lightSprite.width) / 2), y)
end

function drawTinted(ctx::Ahorn.Cairo.CairoContext, blockSprite::Ahorn.Sprite, a::Number, b::Number, c::Number, d::Number, e::Number, f::Number, tint::Ahorn.rgbaTupleType)
	tinted = Ahorn.CairoImageSurface(zeros(UInt32, e, f), Ahorn.Cairo.FORMAT_ARGB32)
    tintedCtx = Ahorn.CairoContext(tinted)
	Ahorn.drawImage(tintedCtx, blockSprite, 0, 0, c, d, e, f)
    tinted = Ahorn.tintSurfaceMultiplicative(tinted, tint)
	Ahorn.drawImage(ctx, tinted, a, b, 0, 0, e, f)
end

function Ahorn.renderAbs(ctx::Ahorn.Cairo.CairoContext, entity::CustomizableZipMover, room::Maple.Room)
    renderCustomizableZipMover(ctx, entity)
end

end