﻿module FakeSpike

using ..Ahorn, Maple

@mapdef Entity "altlevelshelper/fakespikesup" FakeSpikeUp(x::Integer, y::Integer, width::Integer=8, type::String="default")
@mapdef Entity "altlevelshelper/fakespikesdown" FakeSpikeDown(x::Integer, y::Integer, width::Integer=8, type::String="default")
@mapdef Entity "altlevelshelper/fakespikesleft" FakeSpikeLeft(x::Integer, y::Integer, height::Integer=8, type::String="default")
@mapdef Entity "altlevelshelper/fakespikesright" FakeSpikeRight(x::Integer, y::Integer, height::Integer=8, type::String="default")

FakeSpikesUnion = Union{FakeSpikeUp, FakeSpikeDown, FakeSpikeLeft, FakeSpikeRight}

directions = Dict{String, String}(
    "altlevelshelper/fakespikesup" => "up",
    "altlevelshelper/fakespikesdown" => "down",
    "altlevelshelper/fakespikesleft" => "left",
    "altlevelshelper/fakespikesright" => "right",
)

Ahorn.editingOptions(entity::FakeSpikesUnion) = Dict{String, Any}(
    "type" => Maple.spike_types
)

const placements = Ahorn.PlacementDict()

for type in Maple.spike_types
    key = "Fake Spikes (Up) ($(uppercasefirst(type)) (Alt Levels Helper)"
	placements[key] = Ahorn.EntityPlacement(
        FakeSpikeUp,
        "rectangle",
		Dict{String, Any}(
			"type" => type,
		),
    )

    key = "Fake Spikes (Down) ($(uppercasefirst(type)) (Alt Levels Helper)"
	placements[key] = Ahorn.EntityPlacement(
        FakeSpikeDown,
        "rectangle",
		Dict{String, Any}(
			"type" => type,
		),
    )

    key = "Fake Spikes (Left) ($(uppercasefirst(type)) (Alt Levels Helper)"
	placements[key] = Ahorn.EntityPlacement(
        FakeSpikeLeft,
        "rectangle",
		Dict{String, Any}(
			"type" => type,
		),
    )

    key = "Fake Spikes (Right) ($(uppercasefirst(type)) (Alt Levels Helper)"
	placements[key] = Ahorn.EntityPlacement(
        FakeSpikeRight,
        "rectangle",
		Dict{String, Any}(
			"type" => type,
		),
    )
end

offsets = Dict{String, Tuple{Integer, Integer}}(
    "up" => (4, -4),
    "down" => (4, 4),
    "left" => (-4, 4),
    "right" => (4, 4),
)

rotations = Dict{String, Number}(
    "up" => 0,
    "right" => pi / 2,
    "down" => pi,
    "left" => pi * 3 / 2
)

rotationOffsets = Dict{String, Tuple{Number, Number}}(
    "up" => (0.5, 0.25),
    "right" => (1, 0.675),
    "down" => (1.5, 1.125),
    "left" => (0, 1.675)
)

resizeDirections = Dict{String, Tuple{Bool, Bool}}(
    "up" => (true, false),
    "down" => (true, false),
    "left" => (false, true),
    "right" => (false, true),
)

function Ahorn.renderSelectedAbs(ctx::Ahorn.Cairo.CairoContext, entity::FakeSpikesUnion)
    direction = get(directions, entity.name, "up")
    theta = rotations[direction] - pi / 2

    width = Int(get(entity.data, "width", 0))
    height = Int(get(entity.data, "height", 0))

    x, y = Ahorn.position(entity)
    cx, cy = x + floor(Int, width / 2) - 8 * (direction == "left"), y + floor(Int, height / 2) - 8 * (direction == "up")

    Ahorn.drawArrow(ctx, cx, cy, cx + cos(theta) * 24, cy + sin(theta) * 24, Ahorn.colors.selection_selected_fc, headLength=6)
end

function Ahorn.selection(entity::FakeSpikesUnion)
	x, y = Ahorn.position(entity)

	width = Int(get(entity.data, "width", 8))
	height = Int(get(entity.data, "height", 8))

	direction = get(directions, entity.name, "up")

	ox, oy = offsets[direction]

	return Ahorn.Rectangle(x + ox - 4, y + oy - 4, width, height)
end

function Ahorn.minimumSize(entity::FakeSpikesUnion)
	return (8, 8)
end

function Ahorn.resizable(entity::FakeSpikesUnion)
	direction = get(directions, entity.name, "up")

	return resizeDirections[direction]
end

function Ahorn.render(ctx::Ahorn.Cairo.CairoContext, entity::FakeSpikesUnion)
    variant = get(entity.data, "type", "default")
    if haskey(directions, entity.name)
        direction = get(directions, entity.name, "up")
		
        if variant == "tentacles"
			width = get(entity.data, "width", 16)
			height = get(entity.data, "height", 16)

			for ox in 0:16:width - 16, oy in 0:16:height - 16
				drawX, drawY = (ox, oy) .+ (16, 16) .* rotationOffsets[direction]
				Ahorn.drawSprite(ctx, "danger/tentacles00", drawX, drawY, rot=rotations[direction])
			end

			if width / 8 % 2 == 1 || height / 8 % 2 == 1
				drawX, drawY = (width - 16, height - 16) .+ (16, 16)
				Ahorn.drawSprite(ctx, "danger/tentacles00", drawX, drawY, rot=rotations[direction])
			end
        else        
            width = get(entity.data, "width", 8)
            height = get(entity.data, "height", 8)

            for ox in 0:8:width - 8, oy in 0:8:height - 8
                drawX, drawY = (ox, oy) .+ offsets[direction]
                Ahorn.drawSprite(ctx, "danger/spikes/$(variant)_$(direction)00", drawX, drawY)
            end
        end
    end
end

#function Ahorn.render(ctx::Ahorn.Cairo.CairoContext, entity::FakeSpikesUnion)
#        direction = get(directions, entity.name, "up")      
#		width = get(entity.data, "width", 8)
#		height = get(entity.data, "height", 8)
#
#		for ox in 0:8:width - 8, oy in 0:8:height - 8
#			drawX, drawY = (ox, oy) .+ offsets[direction]
#			Ahorn.drawSprite(ctx, "objects/FactoryHelper/FakeSpike/Fake_$(direction)00", drawX, drawY)
#		end
#end

end
